/***************************************************************************/
/*Programmautor: Barbara Unger                                             */
/*Datum:         1997-06-03                                                */
/*Gruppe:        Boerse3                                                   */
/***************************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Date;
import java.io.*;

/* Programm zum Betrachten von Boersenkursen.
   Abfolge der Klassen in dieser Datei:

   interface Konstanten
   abstract class BoersenInfo extends Frame implements Konstanten
   abstract class KostenloseInfos extends BoersenInfo 
   abstract class KostenplichtigeInfos extends BoersenInfo 
   abstract class Chart extends KostenloseInfos 
   class TagesChart extends Chart  
   class TagesChartWert implements Comparable  
   class MonatsChart extends Chart 
   class MonatsChartWert implements Comparable 
   abstract class Tabelle extends KostenloseInfos   
   class TagesTabelle extends Tabelle 
   class Tagesdaten implements Comparable 
   class Zeitraumdaten implements Comparable 
   class Datum 
   class Uhrzeit 
   class Kurs 
   class Tagesgewinne extends Tabelle 
   class LetzteTageskurse extends Tabelle 
   class WochenTabelle extends Tabelle 
   class MonatsTabelle extends Tabelle 
   class MonatsDynamikTabelle extends Tabelle 
   interface Comparable
   abstract class ChartAnzeiger extends Canvas  
   class TagesChartAnzeiger extends ChartAnzeiger 
   class MonatsChartAnzeiger extends ChartAnzeiger 
   class Auswahl extends Frame 
   class Boerse 
*/

interface Konstanten {
//  static Date date = new Date();
//  static Datum heute = new Datum(date.getDay(), date.getMonth(), date.getYear());
  static Datum heute = new Datum(27, 5, 97);
  static String kurse = "KURSE";
  static String kassakurse = "KASSAKURSE";
}


/* abstrakte Klasse BoersenInfo Implementiert Fenster, in denen
   Boersen-Informationen angezeigt werden.  Stellt sicher, dass in
   jeder der daraus abgeleiteten Klassen Knoepfe zum Schliessen des
   Fensters und zum Beenden des Programmes existieren.

   Methoden:
     BoersenInfo(String)     erzeugt das Fenster und setzt den Titel
     schliesseFenster()      kehrt zum aufrufenden Fenster zurueck
     beendeProgramm()        beendet das Programm komplett
*/
   
abstract class BoersenInfo extends Frame implements Konstanten {
  Panel p1 = new Panel();
  Panel p2 = new Panel();
  Panel p3 = new Panel();
  Button schliessen = new Button("Schliessen");
  Button beenden = new Button("Beenden");
  float kosten;

  /*  Konstruktor. Setzt den Titel des Fensters, fuegt drei Panels ein und bestimmt das Layout.
      Panel p1 dient zum Darstellen von Sponsor- oder Kosteninformationen,
      Panel p2 dient zum Anzeigen der angeforderten Informationen,
      Panel p3 enthaelt Knoepfe zum Schliessen des Fensters und zum Beenden des Programms.
  */
  BoersenInfo(String titel) {
    super(titel);
    this.setLayout(new BorderLayout());
    p1.setLayout(new FlowLayout());
    add("North",p1);
    p2.setLayout(new BorderLayout());
    add("Center",p2);
    p3.setLayout(new FlowLayout());
    add("South",p3);
    p3.add(schliessen);
    schliessen.addActionListener(
				 //Die folgende Konstruktion bedeutet, dass eine unbenannte innere
				 //Klasse erzeugt wird. Diese stellt eine fuer das aeussere Objekt
				 //(in diesem Fall der schliessen-Knopf)  spezialisierte
				 //actionPerformed()-Methode zur Verfuegung. Sie wird immer
				 //dann ausgefuehrt, sobald eine Aktion auf dem aeusseren Objekt
				 //ausgefuehrt wird. 
				 new ActionListener(){
				   public void actionPerformed(ActionEvent e){
				     schliesseFenster();
				   }
				 }
				 );
				 
    p3.add(beenden);
    beenden.addActionListener(
			      //Die folgende Konstruktion bedeutet, dass eine unbenannte innere
			      //Klasse erzeugt wird. Diese stellt eine fuer das aeussere Objekt
			      //(in diesem Fall der beenden-Knopf)  spezialisierte
			      //actionPerformed()-Methode zur Verfuegung. Sie wird immer
			      //dann ausgefuehrt, sobald eine Aktion auf dem aeusseren Objekt
			      //ausgefuehrt wird. 
			      new ActionListener(){
				public void actionPerformed(ActionEvent e){
				  beendeProgramm();
				}
			      }
			      );
  }
  
  /* Schliesst das Fenster und gibt seine Ressourcen frei.
   */
  void schliesseFenster() {
    this.dispose();
  }

  /* Beendet das Programm komplett: alle Fenster werden geschlossen.
   */
  void beendeProgramm() {
    System.exit(0);
  }
}


/* abstrakte Klasse KostenloseInfos
   Stellt sicher, dass fuer das Abrufen von Informationen durch alle
   aus ihr abgeleiteten Klassen keine Kosten anfallen. (Kosten werden
   von einem Sponsor getragen).
   Des weiteren stellt sie eine Methode zum Dateneinlesen
   bereit. Welche Daten jedoch daraus erzeugt werden ist zu diesem
   Zeitpunkt unklar und wird erst in einer Unterklasse entschieden
   durch die Methode erzeugeDatenvector().  Ausserdem wird nun schon
   ein Sponsorlabel fuer alle kostenlosen Infos gesetzt.

   Methoden:
   KostenloseInfos(String) Konstruktor, setzt Fenstertitel, fuegt einen Sponsor ein
                       und setzt die Kosten auf 0.00.
   leseDaten(String)  liest Daten aus einer Datei und schreibt sie mit Hilfe
                       der Methode erzeugeDatenvector() in den Vector vec
   qsort(int, int), partition(int, int), exchange(int, int), sortiere()
                       sind Methoden, die einen Sortieralgorithmus fuer alle
		       Unterklassen zur Verfuegung stellen. Da in jeder
		       Unterklasse unterschiedliche Datentypen zu sortieren
		       sind, wurde Schablonenmethode angewandt, die
		       vergleiche()-Methode ist nur abstrakt realisiert
		       und wird vom jeweiligen Datentyp erst implementiert.
*/

abstract class KostenloseInfos extends BoersenInfo {
  String dateiname;
  Label sponsor;
  Vector vec = new Vector();
  
  /* Konstruktor, setzt den Titel des Fensters, fuegt einen Sponsor in das
     Fenster ein (in p1) und setzt die Kosten auf 0.00
   */
  KostenloseInfos(String titel) {
    super(titel);
    sponsor = new Label("Diese Seiten werden gesponsort von xxy");
    p1.add(sponsor);
    kosten = 0.0f;
  }

  /* Liest die gewuenschten Daten aus einer Datei und erzeugt durch die
     abstrakte Methode erzeugeDatenvector eine Datenstruktur, in der die
     relevanten Daten gespeichert werden.
   */
  void leseDaten(String dateiname) {
    try{
      BufferedReader br = new BufferedReader(new FileReader(dateiname));
      String s;
      try{
	while((s = br.readLine())!=null) {
	  if(!s.startsWith("#")) {
	    erzeugeDatenvector(s);
	  }
	}
      } catch (IOException e) {
	System.err.println("Lesefehler bei Datei " + dateiname);
      }
    } catch (FileNotFoundException e) {
      System.err.println("Die Datei " + dateiname + " konnte nicht geoeffnet werden");
    }
  }

  /* Erzeugt aus einem String ein Objekt in dem die relevanten Daten
     gespeichert sind. Dieses Objekt wird im Vektor vec gepeichert
   */
  abstract void erzeugeDatenvector(String s);
  
  /* Beschreibung des Quicksort siehe oben
   */  
  void qsort(int left, int right) {
    int middle;
    if(left < right){
      middle = partition(left, right);
      qsort(left,middle-1);
      qsort(middle+1, right);
    }
  }

  /* Hilfsprozedur fuer den Quicksort
   */
  int partition(int left, int right) {
    Comparable pivot = (Comparable)vec.elementAt(left);
    int l = left;
    int r = right;
    int middle;
    while(l < r) {
      while((l <= right) && (((Comparable)vec.elementAt(l)).vergleiche(pivot))){
	l = l+1;
      }
      while((r >= left) && (!((Comparable)vec.elementAt(r)).vergleiche(pivot))){
	r = r-1;
      }
      if(l < r) {
	exchange(l, r);
      }
    }
    middle = r;
    exchange (left, middle);
    return middle;
  }

  /* Hilfsprozedur fuer den Quicksort
   */
  void exchange(int i, int j) {
    Comparable c = (Comparable)vec.elementAt(i);
    vec.setElementAt(vec.elementAt(j),i);
    vec.setElementAt(c,j);
  }

  /* Ruft Quicksort auf
   */  
  void sortiere() {
    qsort(0, vec.size()-1);
  }
}


/* Klasse KostenpflichtigeInfos
   enthaelt einen Rahmen fuer Informationen, die vom Kunden bezahlt werden
   muessen, wie beispielsweise Prognosen, etc.
 */
/*Wird nicht ausprogrammiert.*/
abstract class KostenplichtigeInfos extends BoersenInfo {

  KostenplichtigeInfos(String titel) {
    super(titel);
    kosten = 1.0f;
  }
}


/* abstrakte Klasse Chart
   Diese Klasse enthaelt allgemeine Methoden zur Darstellung eines Charts.
   Dazu muss erst einmal ein Wertpapier ausgewaehlt werden, das angezeigt
   werden soll, und nach der Auswahl sollte der Chart angezeigt werden.

   Methoden:
   Chart(String)       Konstruktor, setzt den Fenstertitel
   void stelleDar()    stellt eine Dialogbox dar.
*/

abstract class Chart extends KostenloseInfos {
  int xChartSize = 200;
  int yChartSize = 200;
  String wkn;
  Label l = new Label();
  TextField t = new TextField("700045634",9); 
  Button ok = new Button("ok");
  
  /* Konstruktor, ruft den Konstruktor von KostenloseInfos auf, und fuegt in den
     freien Rahmen p2 eine Auswahlmoeglichkeit des gewuenschten Wertpapiers ein
   */
  Chart(String titel) {
    super(titel);
  }

  void stelleDar(){
    p2.add("North",l);
    p2.add("Center",t);
    p2.add("South",ok);
    ok.addActionListener(
			 //Die folgende Konstruktion bedeutet, dass eine unbenannte innere
			 //Klasse erzeugt wird. Diese stellt eine fuer das aeussere Objekt
			 //(in diesem Fall der ok-Knopf)  spezialisierte
			 //actionPerformed()-Methode zur Verfuegung. Sie wird immer
			 //dann ausgefuehrt, sobald eine Aktion auf dem aeusseren Objekt
			 //ausgefuehrt wird. 
			 new ActionListener(){
			   public void actionPerformed(ActionEvent e){
			     wkn = t.getText();
			     wkn.trim();
			     p2.removeAll();
			     leseDaten(dateiname);
			     p2.add("Center",zeigeCanvasAn());
			     pack();
			     show();
			   }
			 }
			 );
    pack();
    show();  
  }

  
/*  liefert ein Canvas zurueck. Was das Canvas enthaelt, wird erst in der Unterklasse
    bestimmt.
 */
  abstract Canvas zeigeCanvasAn();
}


/* Klasse TagesChart
   In dieser Klasse wird das Erzeugen des Datenvektors implementiert.
   Diese entscheidet durch eine selektiere()-Methode, welche
   Datensaetze ueberhaupt ausgewaehlt werden sollen.

   Methoden:
     TagesChart(String)        Konstruktor, der den Titel setzt und die Datei
                                mit den Daten angibt.
     erzeugeDatenvector(String) bekommt eine Datensatz in Stringform und
                                entscheidet mit Hilfe von selektiere(), ob der
				Datensatz in den Datenvektor eingefuegt
				werden soll.
     selektiere(String s)       entscheidet, ob der String ausgewaehlt werden
                                soll, konkret: ob er von dem angegebenen
				Datum ist und die entsprechende
				Wertpapierkennnummer hat
     zeigeCanvasAn()                  erzeugt ein Objekt der Canvas-Klasse, in der der
                                Chart aus den selektierten Daten, die im Datenvektor stehen,
				angezeigt wird.
*/

class TagesChart extends Chart { 
  TagesChartWert tcw;

/* Konstruktor, der den Titel setzt und die Datei mit den Daten angibt
 */
  TagesChart(String titel) {
    super(titel);
    dateiname = kurse;
    l.setText("Bitte geben Sie die Wertpapierkennnummer fuer den Tageschart an");
  }

/* bekommt eine Datensatz in Stringform und entscheidet mit Hilfe von selektiere(),
   ob der Datensatz in den Datenvektor eingefuegt werden soll.
 */
  void erzeugeDatenvector(String s){
    if(selektiere(s) == true) {
      tcw = new TagesChartWert(s);
      vec.addElement(tcw);
    }
  }

/* entscheidet, ob der String ausgewaehlt werden soll, konkret: ob er von dem angegebenen
   Datum ist und die entsprechende Wertpapierkennnummer hat
 */
  boolean selektiere(String s) {
    Datum d = new Datum();
    d.setzeDatum(s.substring(0,8));
    if((heute.tag == d.tag) && (heute.monat ==d.monat) && (heute.jahr == d.jahr)) {
      return (wkn.compareTo(s.substring(15,24))==0);
    }
    return false;
  }

/* erzeugt ein Objekt der Canvas-Klasse, in der der Chart aus den selektierten Daten, die im
   Datenvektor stehen, angezeigt wird.
 */
  Canvas zeigeCanvasAn() {
    return new TagesChartAnzeiger(vec, xChartSize, yChartSize);
  }
}



/*Klasse TagesChartWert
  ist ein Objekttyp, der aus einer Uhrzeit und einem Kurswert besteht.
  Das sind genau die Daten, die man zum Anzeigen eines Tagescharts braucht. Die Klasse hat
  eine vergleiche-Methode, so dass man einen Vector mit Objekten dieses Typs mit
  der sortier()-Methode aus der Klasse KostenloseInfos sortieren kann.

  Methoden:
    TagesChartWert(String) bekommt einen String, in dem die Uhrzeit
                             und der Kurs enthalten sind.
    vergleiche(Comparable c) vergleicht die Uhrzeiten                
*/

class TagesChartWert implements Comparable { 
  Uhrzeit uhrzeit;
  Kurs kurs;

/* bekommt einen String, in dem die Uhrzeit und der Kurs enthalten sind.
 */
  TagesChartWert(String s) {
    uhrzeit = new Uhrzeit(s.substring(9,15));
    kurs = new Kurs(s.substring(s.length()-7, s.length()));
  }

/* vergleicht die Uhrzeiten zweier TagesChartWert-Objekte
 */
  public boolean vergleiche(Comparable c) {
    return ((this.uhrzeit).kleinergleich(((TagesChartWert)c).uhrzeit));
  }
}


/* Klasse MonatsChart
   In dieser Klasse wird das Erzeugen des Datenvectors implementiert.
   Diese entscheidet durch eine selektiere()-Methode, welche
   Datensaetze ueberhaupt ausgewaehlt werden sollen.

   Methoden:
     MonatsChart(String)       Konstruktor, der den Titel setzt und die Datei
                                mit den Daten angibt.
     erzeugeDatenvector(String) bekommt eine Datensatz in Stringform und
                                entscheidet mit Hilfe von slektiere(), ob der
				Datensatz in den Datenvector eingefuegt
				werden soll.
     selektiere(String s)       entscheidet, ob der String ausgewaehlt werden
                                soll, konkret: ob er von dem angegebenen
				Datum ist und die entsprechende
				Wertpapierkennnummer hat
     zeigeCanvasAn()                 erzeugt ein Objekt der Canvas-Klasse, in der der
                                Chart angezeigt wird.
*/

class MonatsChart extends Chart {
  MonatsChartWert mcw;
  Datum letzterMonat = new Datum();
  
  MonatsChart(String titel) {
    super(titel);
    l.setText("Bitte geben Sie die Wertpapierkennnummer fuer den Monatschart an");
    dateiname = kassakurse;
    letzterMonat = heute.monatzurueck();
  }  

  void erzeugeDatenvector(String s){
    if(selektiere(s) == true) {
      mcw = new MonatsChartWert(s);
      vec.addElement(mcw);
    }
  }

  boolean selektiere(String s) {
    Datum d = new Datum();
    d.setzeDatum(s.substring(0,8));
    if(!d.kleinergleich(letzterMonat)) {
      return (wkn.compareTo(s.substring(9,18))==0);
    }
    return false;
  }

  Canvas zeigeCanvasAn() {
    return new MonatsChartAnzeiger(vec, xChartSize, yChartSize);
  }
}


/*Klasse MonatsChartWert
  ist ein Objekttyp, der aus einem Datum und einem Kassakurswert besteht,
  Daten, die man zum Anzeigen eines Monatschart braucht. Die Klasse hat
  eine vergleiche-Methode, so dass man einen Vector mit diesem Typ mit
  der sortier()-Methode aus der Klasse KostenloseInfos sortieren kann.

  Methoden:
    MonatsChartWert(String) bekommt einen String, in dem das Datum
                                  und der Kurs enthalten sind.
    vergleiche(Comparable c)      vergleicht das Datum                
*/

class MonatsChartWert implements Comparable {
  Kurs kassakurs;
  Datum datum;

  MonatsChartWert(String s){
    kassakurs = new Kurs(s.substring(s.length()-6, s.length()));
    datum = new Datum(s.substring(0,9));
  }

  public boolean vergleiche(Comparable c){
    return ((this.datum).kleinergleich(((MonatsChartWert)c).datum));
  }    
}


/* abstrakte Klasse Tabelle.
   Gemeinsame Oberklasse fuer alle tabellarischen Darstellungen von
   Kursen oder Kassakursen.
*/

abstract class Tabelle extends KostenloseInfos {  
  String dateiname;

  Tabelle(String titel) {
    super(titel);
    setzeDateiname();
  }

  /* Diese Methoden koennen nicht im Konstruktor aufgerufen werden, da
     zum Zeitpunkt der Ausfuehrung des Konstruktors manche Variablen
     noch nicht gesetzt wurden. Daher werden sie in eine eigene Methode
     "stelleDar" aufgerufen.  
   */ 
  void stelleDar(){
    leseDaten(dateiname);
    zeigeTabelleAn();
  }

  abstract void setzeDateiname();
  abstract void zeigeTabelleAn();
}

/* Klasse zur tabellarischen Darstellung aller Kurse eines Tages
*/

class TagesTabelle extends Tabelle {
  TextArea textarea;

  TagesTabelle(String titel) {
    super(titel);
  }

  void setzeDateiname(String s) {
    dateiname = s;
  }
  
  void setzeDateiname() {
    dateiname = kurse;
  }
  
  boolean selektiere(String s) {
    Datum d = new Datum();
    d.setzeDatum(s.substring(0,8));
    return ((heute.tag == d.tag) && (heute.monat == d.monat) && (heute.jahr == d.jahr));
  }
    

  void erzeugeDatenvector(String s) {
    if(selektiere(s) == true) {
      Datum datum = new Datum(s.substring(0,8));
      Uhrzeit uhrzeit = new Uhrzeit(s.substring(9,14));
      Kurs kurs = new Kurs(s.substring(s.length() - 7, s.length()));
      String wkn = s.substring(14,24);
      String name = s.substring(25, s.length()-7);
      Tagesdaten tagesdaten = new Tagesdaten(datum, uhrzeit, wkn, name, kurs);
      vec.addElement(tagesdaten);
    }
  }
  
  void zeigeTabelleAn(){
    textarea = new TextArea(20,40);
    p2.add("Center",textarea);
    sortiere();
    for(int i = 0; i < vec.size(); i++) {
      textarea.append((((Tagesdaten)vec.elementAt(i)).uhrzeit).stunde + ":" +
		      (((Tagesdaten)vec.elementAt(i)).uhrzeit).minute + " " +
		      ((Tagesdaten)vec.elementAt(i)).wertpapierkennnummer + " " +
		      ((Tagesdaten)vec.elementAt(i)).name + " " +
		      ((Tagesdaten)vec.elementAt(i)).kurs.kurs + "\n"); 
    }
    if(vec.size() == 0) {
      textarea.append("Fuer das heutige Datum, " + heute.tag + "." +
		      heute.monat + "." + heute.jahr  +
		      ", existieren keine Daten\n");
    }
    this.pack();
    this.show();
  }
}


/* Klasse zur Repraesentation eines einzelnen Kursinformations-Datensatzes
*/

class Tagesdaten implements Comparable {
  Datum datum;
  Uhrzeit uhrzeit;
  String wertpapierkennnummer;
  String name;
  Kurs kurs;
  
  Tagesdaten(Datum d, Uhrzeit u, String wkn, String n, Kurs k) {
    datum = d;
    uhrzeit = u;
    wertpapierkennnummer = wkn;
    name = n;
    kurs = k;
  }

  public boolean vergleiche(Comparable c) {
    return (this.uhrzeit).kleinergleich(((Tagesdaten)c).uhrzeit);
  }
}


/* Klasse zur Repraesentation eines einzelnen Kassakursinformations-Datensatzes
*/

class Zeitraumdaten implements Comparable {
  Datum datum;
  String wertpapierkennnummer;
  String name;
  Kurs kassakurs;

  Zeitraumdaten(Datum d, String wkn, String n, Kurs k) {
    datum = d;
    wertpapierkennnummer = wkn;
    name = n;
    kassakurs = k;
  }

  public boolean vergleiche(Comparable c) {
    return (this.datum).kleinergleich(((Zeitraumdaten)c).datum);
  }
}


/* Klasse zur Repraesentation eines Datums, incl. Berechnung des
   Datums 'vor einer Woche' und 'vor einem Monat'.
   (Anmerkung zum Experiment:
    Diese Routinen sind vereinfacht, um den Code zu verkuerzen.
    Sie funktionieren nicht wirklich korrekt. Das soll uns hier aber mal
    gerade nicht stoeren...)
*/

class Datum {
  int tag;
  int monat;
  int jahr;

  Datum() {
    tag = 0;
    monat = 0;
    jahr = 0;
  }
  
  Datum(String s){
    tag = new Integer(s.substring(0,2)).intValue();
    monat = new Integer(s.substring(3,5)).intValue();
    jahr = new Integer(s.substring(6,8)).intValue();
  }

  Datum(int t, int m, int j){
    tag = t;
    monat = m;
    jahr = j;
  }

  void setzeDatum(String s) {
    tag = new Integer(s.substring(0,2)).intValue();
    monat = new Integer(s.substring(3,5)).intValue();
    jahr = new Integer(s.substring(6,8)).intValue();
  }

  void setzeDatum(int t, int m, int j) {
    tag = t;
    monat = m;
    jahr = j;
  }

  boolean kleinergleich(Datum d) {
    if(this.jahr < d.jahr) {
      return true;
    } else if(this.jahr == d.jahr){
      if(this.monat < d.monat) {
	return true;
      } else if(this.monat == d.monat){
	if(this.tag <= d.tag) {
	  return true;
	} 
      }
    }
    return false;
  }
 
  Datum monatzurueck() {
    Datum d = new Datum();
    d.tag = tag;
    d.jahr = jahr;
    if(monat == 1) {
      d.monat = 12;
      d.jahr = jahr - 1;
    } else {
      d.monat = monat - 1;
    }
    return d;
  }

  /* vereinfachte Berechnung: Alle Monate haben 30 Tage
   */
  Datum wochezurueck() {
    Datum d = new Datum();
    d.tag = tag - 7;
    d.monat = monat;
    d.jahr = jahr;
    if(d.tag < 1) {
      d.tag = d.tag + 30;
      d.monat = monat - 1;
      if(d.monat < 1) {
	d.monat = d.monat + 12;
	d.jahr = jahr - 1;
      }
    }
    return d;
  }

  void print() {
    System.out.println(" Datum = " + tag + "." + monat + "." + jahr);
  }
}

/* Klasse zur Repraesentation einer Uhrzeit
*/

class Uhrzeit {
  int stunde;
  int minute;

  Uhrzeit(String s) {
    stunde = (new Integer(s.substring(0,2))).intValue();
    minute = (new Integer(s.substring(3,5))).intValue();
  }

  boolean kleinergleich(Uhrzeit u) {
    if(this.stunde < u.stunde) {
      return true;
    } else if(this.stunde == u.stunde) {
      if(this.minute <= u.minute) {
	return true;
      }
    }
    return false;
  }
}

/* Klasse zur Repraesentation eines Kurses oder Kassakurses
*/

class Kurs {
  float kurs;

  Kurs(String s){
    kurs = (new Float(s)).floatValue();
  }
}


/* Tabellarische Darstellung der prozentualen Unterschiede zwischen dem
   ersten und letzten Kurs jeder Aktie am heutigen Datum.
*/

class Tagesgewinne extends Tabelle {
  TextArea textarea;
  Vector v = new Vector();
  
  Tagesgewinne(String titel) {
    super(titel);
  }
  
  void setzeDateiname(String s) {
    dateiname = s;
  }
  
  void setzeDateiname() {
    dateiname = kurse;
  }
  
  boolean selektiere(String s) {
    Datum d = new Datum();
    d.setzeDatum(s.substring(0,8));
    return ((heute.tag == d.tag) && (heute.monat == d.monat) && (heute.jahr == d.jahr));
  }
    
  void erzeugeDatenvector(String s) {
    if(selektiere(s) == true) {
      Datum datum = new Datum(s.substring(0,8));
      Uhrzeit uhrzeit = new Uhrzeit(s.substring(9,14));
      Kurs kurs = new Kurs(s.substring(s.length() - 7, s.length()));
      String wkn = s.substring(14,24);
      String name = s.substring(25, s.length()-7);
      Tagesdaten tagesdaten = new Tagesdaten(datum, uhrzeit, wkn, name, kurs);
      vec.addElement(tagesdaten);
    }
  }
  
  boolean schonAbgearbeitet(String s) {
    int i = 0;
    while(i < v.size()) {
      if(((String)v.elementAt(i)).compareTo(s)==0) {
	i = v.size();
	return true;
      } else {
	i++;
      }
    }
    v.addElement(s);
    return false;
  }

  float tagesgewinn(String wertpapierkennnummer, int start) {
    float min = 0.0f;
    float max = 0.0f;
    int i = start;
    while((wertpapierkennnummer.compareTo(((Tagesdaten)(vec.elementAt(i))).wertpapierkennnummer) != 0) && (i < vec.size())) {
      i = i+1;
    }
    min = ((Tagesdaten)(vec.elementAt(i))).kurs.kurs;
    while(i  < vec.size()) {
      if(wertpapierkennnummer.compareTo(((Tagesdaten)(vec.elementAt(i))).wertpapierkennnummer) == 0) {
	max = ((Tagesdaten)(vec.elementAt(i))).kurs.kurs;
      }
      i = i+1;	
    }
    //    System.out.println("Min = " + min + "     Max = " + max); 
    if(min == 0.0f) {
      return 0;
    } else {
      return ((max - min)/min);
    }
  }

  void zeigeTabelleAn() {
    textarea = new TextArea(20,40);
    p2.add("Center",textarea);
    boolean keineDaten = true;
    sortiere();
    for(int i = 0; i < vec.size(); i++) {
      if(! schonAbgearbeitet(((Tagesdaten)(vec.elementAt(i))).wertpapierkennnummer)) {
	textarea.append(((Tagesdaten)(vec.elementAt(i))).name + " " +
			((Tagesdaten)(vec.elementAt(i))).wertpapierkennnummer + " "
			+ tagesgewinn(((Tagesdaten)(vec.elementAt(i))).wertpapierkennnummer, i) +
			"\n");
	keineDaten = false;
      }
    }
    if(keineDaten == true) {
      textarea.append("Fuer das heutige Datum, " + heute.tag + "." +
		      heute.monat + "." + heute.jahr  +
		      ", existieren keine Daten\n");
    }
    this.pack();
    this.show();
  }
}

/* tabellarische Darstellung der letzten vorhandenen heutigen Kurse fuer alle 
   Aktientitel.
*/

class LetzteTageskurse extends Tabelle {
  TextArea textarea;
  Vector v = new Vector();
  
  LetzteTageskurse(String titel) {
    super(titel);
  }

  void setzeDateiname(String s) {
    dateiname = s;
  }
  
  void setzeDateiname() {
    dateiname = kurse;
  }
  
  boolean selektiere(String s) {
    Datum d = new Datum();
    d.setzeDatum(s.substring(0,8));
    return ((heute.tag == d.tag) && (heute.monat == d.monat) && (heute.jahr == d.jahr));
  }
    
  void erzeugeDatenvector(String s) {
    if(selektiere(s) == true) {
      Datum datum = new Datum(s.substring(0,8));
      Uhrzeit uhrzeit = new Uhrzeit(s.substring(9,14));
      Kurs kurs = new Kurs(s.substring(s.length() - 7, s.length()));
      String wkn = s.substring(14,24);
      String name = s.substring(25, s.length()-7);
      Tagesdaten tagesdaten = new Tagesdaten(datum, uhrzeit, wkn, name, kurs);
      vec.addElement(tagesdaten);
    }
  }
  
  boolean schonAbgearbeitet(String s) {
    int i = 0;
    while(i < v.size()) {
      if(((String)v.elementAt(i)).compareTo(s)==0) {
	i = v.size();
	return true;
      } else {
	i++;
      }
    }
    v.addElement(s);
    return false;
  }

  float letztetageskurse(String wertpapierkennnummer, int start) {
    float letzter = 0.0f;
    int i = start;
    while(i < vec.size()) {
      if(wertpapierkennnummer.compareTo(
           ((Tagesdaten)(vec.elementAt(i))).wertpapierkennnummer) == 0) {
        letzter = ((Tagesdaten)(vec.elementAt(i))).kurs.kurs;
      }
      i = i+1;	
    }
    return letzter;
  }

  void zeigeTabelleAn() {
    textarea = new TextArea(20,40);
    p2.add("Center",textarea);
    sortiere();
    for(int i = 0; i < vec.size(); i++) {
      if(!schonAbgearbeitet(
             ((Tagesdaten)(vec.elementAt(i))).wertpapierkennnummer)) {
	textarea.append(((Tagesdaten)(vec.elementAt(i))).name + " " +
			((Tagesdaten)(vec.elementAt(i))).wertpapierkennnummer + " " +
			letztetageskurse(
			((Tagesdaten)(vec.elementAt(i))).wertpapierkennnummer, i) +
			"\n");
      }
    }
    if(vec.size() == 0) {
      textarea.append("Fuer das heutige Datum, " + 
		      heute.tag + "." + heute.monat + "." +
		      heute.jahr  + ", existieren keine Daten\n");
    }
    this.pack();
    this.show();
  }
}


/* tabellarische Darstellung der Kassakurse aller Aktientitel fuer die
   letzten 7 Tage. Benutzt die Kassakurse.
*/

class WochenTabelle extends Tabelle {
  Datum vonDatum;
  TextArea textarea;

  WochenTabelle(String titel) {
    super(titel);
    vonDatum = heute.wochezurueck();
  }

  void setzeDateiname(String s) {
    dateiname = s;
  }
  
  void setzeDateiname() {
    dateiname = kassakurse;
  }
  
  boolean selektiere(String s) {
    Datum d = new Datum();
    d.setzeDatum(s.substring(0,8));
    return (!d.kleinergleich(vonDatum));
  }
  
  void erzeugeDatenvector(String s){
    if(selektiere(s) == true) {
      Datum datum = new Datum(s.substring(0,8));
      String wkn = s.substring(9,18);
      String name = s.substring(19, s.length()-7);
      Kurs kassakurs = new Kurs(s.substring(s.length()-6,s.length()));
      Zeitraumdaten zrd = new Zeitraumdaten(datum, wkn, name, kassakurs);
      vec.addElement(zrd);
    }
  }

  void zeigeTabelleAn(){
    textarea = new TextArea(20,40);
    p2.add("Center",textarea);
    sortiere();
    for(int i = 0; i < vec.size(); i++) {
      textarea.append((((Zeitraumdaten)vec.elementAt(i)).datum).tag + "." +
		      (((Zeitraumdaten)vec.elementAt(i)).datum).monat + "." +
		      (((Zeitraumdaten)vec.elementAt(i)).datum).jahr + " " +
		      ((Zeitraumdaten)vec.elementAt(i)).wertpapierkennnummer + " " +
		      ((Zeitraumdaten)vec.elementAt(i)).name + " " +
		      ((Zeitraumdaten)vec.elementAt(i)).kassakurs.kurs + "\n"); 
    }
    if(vec.size() == 0) {
      textarea.append("Fuer der gewuenschten Zeitraum existieren keine Daten\n");
    }
    this.pack();
    this.show();
  }
}


/* tabellarische Darstellung der Kassakurse aller Aktientitel fuer die
   letzten 30 Tage (genauer gesagt: seit dem gleichen Tag im Vormonat).
   Benutzt die Kassakurse.
*/

class MonatsTabelle extends Tabelle {
  Datum vonDatum;
  TextArea textarea;
 
  MonatsTabelle(String titel) {
    super(titel);
    vonDatum = heute.monatzurueck();
  }

  void setzeDateiname(String s) {
    dateiname = s;
  }
  
  void setzeDateiname() {
    dateiname = kassakurse;
  }
  
  boolean selektiere(String s) {
    Datum d = new Datum();
    d.setzeDatum(s.substring(0,8));
    return (!d.kleinergleich(vonDatum));
  }
  
  void erzeugeDatenvector(String s){
    if(selektiere(s) == true) {
      Datum datum = new Datum(s.substring(0,8));
      String wkn = s.substring(9,18);
      String name = s.substring(19, s.length()-7);
      Kurs kassakurs = new Kurs(s.substring(s.length()-6,s.length()));
      Zeitraumdaten zrd = new Zeitraumdaten(datum, wkn, name, kassakurs);
      vec.addElement(zrd);
    }
  }

  void zeigeTabelleAn(){
    textarea = new TextArea(20,40);
    p2.add("Center",textarea);
    sortiere();
    for(int i = 0; i < vec.size(); i++) {
      textarea.append((((Zeitraumdaten)vec.elementAt(i)).datum).tag + "." +
		      (((Zeitraumdaten)vec.elementAt(i)).datum).monat + "." +
		      (((Zeitraumdaten)vec.elementAt(i)).datum).jahr + " " +
		      ((Zeitraumdaten)vec.elementAt(i)).wertpapierkennnummer + " " +
		      ((Zeitraumdaten)vec.elementAt(i)).name + " " +
		      ((Zeitraumdaten)vec.elementAt(i)).kassakurs.kurs + "\n"); 
    }
    if(vec.size() == 0) {
      textarea.append("Fuer der gewuenschten Zeitraum existieren keine Daten\n");
    }
    this.pack();
    this.show();
  }
}


/* tabellarische Darstellung der prozentualen Unterschiede zwischen
   aufeinanderfolgenden Kassakursen aller Aktientitel fuer den
   letzten Monat. Benutzt die Kassakurse.
*/

class MonatsDynamikTabelle extends Tabelle {
  Vector dyn = new Vector();
  Datum vonDatum;
  TextArea textarea;
  
  
  MonatsDynamikTabelle(String titel) {
    super(titel);
    vonDatum = heute.monatzurueck();
  }

  void setzeDateiname(String s) {
    dateiname = s;
  }
  
  void setzeDateiname() {
    dateiname = kassakurse;
  }
  
  boolean selektiere(String s) {
    Datum d = new Datum();
    d.setzeDatum(s.substring(0,8));
    return (!d.kleinergleich(vonDatum));
  }
  
  void erzeugeDatenvector(String s){
    if(selektiere(s) == true) {
      Datum datum = new Datum(s.substring(0,8));
      String wkn = s.substring(9,18);
      String name = s.substring(19, s.length()-7);
      Kurs kassakurs = new Kurs(s.substring(s.length()-6,s.length()));
      Zeitraumdaten zrd = new Zeitraumdaten(datum, wkn, name, kassakurs);
      vec.addElement(zrd);
    }
  }

  float berechneDynamik(int i) {
    //suche den letzen Wert des gleichen Wertpapiers, um die prozentuale
    //Abweichung des neuen Wertes vom alten zu bestimmen.
    int j = i-1;
    boolean gefunden = false;
    while ((j >= 0) && !gefunden) {
      if ((((Zeitraumdaten)vec.elementAt(i)).wertpapierkennnummer).compareTo(
           ((Zeitraumdaten)vec.elementAt(j)).wertpapierkennnummer) == 0) {
	gefunden = true;
	return ((((((Zeitraumdaten)vec.elementAt(i)).kassakurs).kurs - 
                  (((Zeitraumdaten)vec.elementAt(j)).kassakurs).kurs)/
                   (((Zeitraumdaten)vec.elementAt(j)).kassakurs).kurs)*100);
      }
      j--;
    }
    return 0.0f;
  }
      
  void zeigeTabelleAn(){
    textarea = new TextArea(20,40);
    p2.add("Center",textarea);
    sortiere();
    for(int i = 0; i < vec.size(); i++) {
      textarea.append((((Zeitraumdaten)vec.elementAt(i)).datum).tag + "." +
		      (((Zeitraumdaten)vec.elementAt(i)).datum).monat + "." +
		      (((Zeitraumdaten)vec.elementAt(i)).datum).jahr + " " +
		      ((Zeitraumdaten)vec.elementAt(i)).wertpapierkennnummer + " " +
		      ((Zeitraumdaten)vec.elementAt(i)).name + "      " +
		      berechneDynamik(i) + "%\n");
    }
    if(vec.size() == 0) {
      textarea.append("Fuer diesen Zeitraum existieren keine Daten\n");
    }
    this.pack();
    this.show();
  }
}


/* Schnittstelle fuer 'vergleichbare' Datentypen (zum Sortieren).
   a.vergleiche(b) liefert 'a kleiner-oder-gleich b'
*/

interface Comparable {
  public boolean vergleiche(Comparable c);
}

/* Oberklasse aller graphischen Anzeigeklassen
*/

abstract class ChartAnzeiger extends Canvas { 
  Vector v;
  int xSize;
  int ySize;
  int abstand;
  
  ChartAnzeiger(Vector vec, int xSize, int ySize) {
    v = vec;
    this.xSize = xSize;
    this.ySize = ySize;
    if(vec.size() != 0) {
      abstand = xSize/vec.size();
    }
    setSize(xSize, ySize);
  }
      
  public void paint(Graphics g) {
    g.drawRect(0, 0, xSize-1, ySize-1);
    for(int i = 0; i < v.size()-1; i++) {
      drawLine(g, i);
    }
  }

  abstract void drawLine(Graphics g, int i);
}


/* graphische Darstellung aller Kurse eines Aktientitels fuer den heutigen
   Tag.
*/

class TagesChartAnzeiger extends ChartAnzeiger {

  TagesChartAnzeiger(Vector vec, int xSize, int ySize) {
    super(vec, xSize, ySize);
  }

  void drawLine(Graphics g, int i){
    g.drawLine(i*abstand,
	       ySize - (int)((((TagesChartWert)(v.elementAt(i))).kurs).kurs),
	       (i+1)*abstand,
	       ySize - (int)((((TagesChartWert)(v.elementAt(i+1))).kurs).kurs));
  }      
}

  
/* graphische Darstellung aller Kassakurse eines Aktientitels fuer die
   letzten 30 Tage.
*/

class MonatsChartAnzeiger extends ChartAnzeiger {
  
  MonatsChartAnzeiger(Vector vec, int xSize, int ySize) {
    super(vec, xSize, ySize);
  }

  void drawLine(Graphics g, int i){
    g.drawLine(i*abstand,
	       ySize - (int)((((MonatsChartWert)(v.elementAt(i))).kassakurs).kurs),
	       (i+1)*abstand,
	       ySize - (int)((((MonatsChartWert)(v.elementAt(i+1))).kassakurs).kurs));
  }      
}

/* Kern des Hauptprogramms: Auswahlmenue der einzelnen Darstellungsformen.
*/

class Auswahl extends Frame {
  Button beenden = new Button("Beenden");
  Button ok = new Button("OK");
  Panel p = new Panel();
  CheckboxGroup cbg = new CheckboxGroup();
  Checkbox cbtt = new Checkbox("Tagestabelle", cbg, true);
  Checkbox cbwt = new Checkbox("Wochentabelle", cbg, false);
  Checkbox cbmt = new Checkbox("Monatstabelle", cbg, false);
  Checkbox cbdt = new Checkbox("Dynamiktabelle", cbg, false);
  Checkbox cbtg = new Checkbox("Tagesgewinne", cbg, false);
  Checkbox cbltk = new Checkbox("letzte Tageskurse", cbg, false);
  Checkbox cbtc = new Checkbox("Tages-Chart", cbg, false);
  Checkbox cbmc = new Checkbox("Monats-Chart", cbg, false);
  
  Auswahl(String titel) {
    super(titel);
    setLayout(new BorderLayout());
    p.setLayout(new GridLayout(6,1));
    p.add(cbtt);
    p.add(cbwt);
    p.add(cbmt);
    p.add(cbdt);
    p.add(cbtg);
    p.add(cbltk);
    p.add(cbtc);
    p.add(cbmc);
    add("Center",p);
    add("East",ok);
    ok.addActionListener(
			 //Die folgende Konstruktion bedeutet, dass eine unbenannte innere
			 //Klasse erzeugt wird. Diese stellt eine fuer das aeussere Objekt
			 //(in diesem Fall der ok-Knopf)  spezialisierte
			 //actionPerformed()-Methode zur Verfuegung. Sie wird immer
			 //dann ausgefuehrt, sobald eine Aktion auf dem aeusseren Objekt
			 //ausgefuehrt wird. 
			 new ActionListener(){
			   public void actionPerformed(ActionEvent e){
			     if(cbg.getSelectedCheckbox() == cbtt) {
			       TagesTabelle tt = new TagesTabelle("Tagestabelle");
			       tt.stelleDar();
			     } else if(cbg.getSelectedCheckbox() == cbwt) {
			       WochenTabelle wt = new WochenTabelle("Wochentabelle");
			       wt.stelleDar();
			     } else if(cbg.getSelectedCheckbox() == cbmt) {
			       MonatsTabelle mt = new MonatsTabelle("Monatstabelle");
			       mt.stelleDar();
			     } else if(cbg.getSelectedCheckbox() == cbdt) {
			       MonatsDynamikTabelle dt = new MonatsDynamikTabelle("Dynamiktabelle");
			       dt.stelleDar();
			     } else if(cbg.getSelectedCheckbox() == cbtg) {
			       Tagesgewinne tg = new Tagesgewinne("Tagesgewinne");
			       tg.stelleDar();
			     } else if(cbg.getSelectedCheckbox() == cbltk) {
			       LetzteTageskurse ltg = new LetzteTageskurse("letzte Tageskurse");
			       ltg.stelleDar();
			     } else if(cbg.getSelectedCheckbox() == cbtc) {
			       TagesChart tc = new TagesChart("Tageschart");
			       tc.stelleDar();
			     } else if(cbg.getSelectedCheckbox() == cbmc) {
			       MonatsChart mc = new MonatsChart("Monatschart");
			       mc.stelleDar();
			     }
			   }
			 }
			 );
    add("South",beenden);
    beenden.addActionListener(
			      //Die folgende Konstruktion bedeutet, dass eine unbenannte innere
			      //Klasse erzeugt wird. Diese stellt eine fuer das aeussere Objekt
			      //(in diesem Fall der beenden-Knopf)  spezialisierte
			      //actionPerformed()-Methode zur Verfuegung. Sie wird immer
			      //dann ausgefuehrt, sobald eine Aktion auf dem aeusseren Objekt
			      //ausgefuehrt wird. 
			      new ActionListener(){
				public void actionPerformed(ActionEvent e){
				  System.exit(0);
				}
			      }
			      );
    pack();
    show();
  }
}


/* Programmeinstiegspunkt
*/

class Boerse {
  public static void main(String args[]) {
    Auswahl a = new Auswahl("Boersen-Information");
  }
}

